unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, VirtualPianoKeyboard, Midiout, ExtCtrls, MMSystem;

type
  TFMain = class(TForm)
    VirtualPianoKeyboard1: TVirtualPianoKeyboard;
    Timer1: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure VirtualPianoKeyboard1KeyClick(Sender: TObject; Octave, Note,
      Tone: Integer);
    procedure FormDestroy(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure VirtualPianoKeyboard1TrackMouse(Sender: TObject;
      Keys: Boolean);
  private
    FMidiout: TMidiOutput;
    FMidiOutOpened: boolean;
    FCursorName: string;
    FNotes: TList;
    FDurations: TList;
    procedure SetKeyboardCursor(AName: string = '');
  public
    { Public declarations }
  end;

var
  FMain: TFMain;

implementation

{$R *.dfm}
{$R *.DCR}

var

  NotePitch: array [0..6] of byte = (0,2,4,5,7,9,11);

procedure TFMain.FormCreate(Sender: TObject);
begin
  FMidiout := TMidiOutput.Create(Self);
  FMidiout.Open;
  FMidiOutOpened := FMidiOut.Error = MMSYSERR_NOERROR;
  FNotes := TList.Create;
  FDurations := TList.Create;
end;

procedure TFMain.VirtualPianoKeyboard1KeyClick(Sender: TObject; Octave,
  Note, Tone: Integer);
begin
  if FMidiOutOpened then
  begin
    FMidiout.PutShort($90, Octave * 12 + NotePitch[Note - 1] + Tone, 64);
    FNotes.Add(Pointer(Octave * 12 + NotePitch[Note - 1] + Tone));
    FDurations.Add(Pointer(1 shl (9 - VirtualPianoKeyboard1.DurationPosition)));
  end;
end;

procedure TFMain.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FNotes);
  FreeAndNil(FDurations);
end;

procedure TFMain.Timer1Timer(Sender: TObject);
var
  i: integer;
begin
  for i := FNotes.Count - 1 downto 0 do
  begin
    if integer(FDurations[i]) <= 0 then
    begin
      FMidiout.PutShort($80, integer(FNotes[i]), 0);
      FNotes.Delete(i);
      FDurations.Delete(i);
    end else FDurations[i] := Pointer(integer(FDurations[i]) - 16);
  end;

end;

procedure TFMain.SetKeyboardCursor(AName: string = '');
begin
  if (AName <> '') and (AName <> FCursorName) then
  begin
    Screen.Cursors[5] := Windows.LoadCursor(hInstance, PChar(AName));
    VirtualPianoKeyboard1.Cursor := 5;
    FCursorName := AName;
  end else if AName = '' then
    VirtualPianoKeyboard1.Cursor := crDefault;
end;

procedure TFMain.VirtualPianoKeyboard1TrackMouse(Sender: TObject;
  Keys: Boolean);
begin
  if Keys then
    case VirtualPianoKeyboard1.DurationPosition of
      0: SetKeyboardCursor('KNOTE');
      1: SetKeyboardCursor('KNOTE2');
      2: SetKeyboardCursor('KNOTE4');
      3: SetKeyboardCursor('KNOTE8');
      4: SetKeyboardCursor('KNOTE16');
      5: SetKeyboardCursor('KNOTE32');
      6: SetKeyboardCursor('KNOTE64')
    end
  else SetKeyboardCursor;
end;

end.
